% phase common input model
w1 = 1;
w2 = 0.5;
A = 0.25:0.25:1;
A1 = A;
A2 = A;
O = 0.75;

N = 5000;
L = 5000; % to be discarded
dt = 0.01;
x0 = [0;0;0];

figure
for i = 1:length(A)
x = phasemodel(w1,w2,A1(i),A2(i),O, L, dt, x0);
x0 = x(:,end); % discard
x = phasemodel(w1,w2,A1(i),A2(i),O, N, dt, x0);
t = (0:N-1)*dt;

subplot(length(A),1,i)
plot(t,sin(x(1,:)))
hold on
plot(t,sin(x(2,:)))
hold off
box off
ylabel('sin(x)')
if i~=length(A)
    set(gca,'xticklabel',[])
else
    xlabel('Time')
end
end

w1 = 1;
w2 = 0.5;
O = (w1 + w2)/2;

A = 0.25:0.0001:3;

sin_phi1 = (O - w1) ./ A;
sin_phi2 = (O - w2) ./ A;

cos_phi1 = sqrt(1 - sin_phi1 .^ 2);
cos_phi2 = sqrt(1 - sin_phi2 .^ 2);

sin_delta = sin_phi1 .* cos_phi2 - sin_phi2 .* cos_phi1;

delta = asin(-sin_delta);
delta(A<1/2/sqrt(2)) = pi - delta(A<1/2/sqrt(2));


figure
plot(A, delta)
xlabel('A')
ylabel('delta phase')

function x = phasemodel(w1,w2,A1,A2,O, N, dt, x0)

x = zeros(3,N);
x(:,1) = x0;
for i = 2:N
    k1 = fc(w1,w2,A1,A2,O,x(:,i-1));
    k2 = fc(w1,w2,A1,A2,O,x(:,i-1)+dt/2*k1);
    k3 = fc(w1,w2,A1,A2,O,x(:,i-1)+dt/2*k2);
    k4 = fc(w1,w2,A1,A2,O,x(:,i-1)+dt*k3);
    x(:,i) = x(:,i-1) + (k1 + k2*2 + k3*2 + k4)*dt/6;
end
end

function r = fc(w1,w2,A1,A2,O,x)

r1 = w1 + A1*sin(x(3) - x(1));
r2 = w2 + A2*sin(x(3) - x(2));
r3 = O;
r = [r1;r2;r3];
end