% plot in vitro data of excised marmoset larynx

infant_data = xlsread('invitrodata.xlsx','infant_larynx');

adult_data = xlsread('invitrodata.xlsx','adult_larynx');


%%
% plot f0 map
figure(1)
subplot(1,2,1)
sx = 0.75; % moving average factor
sy = 0.5;
[edgeXi, edgeYi, Ni] = nmeanhist_smooth(infant_data(:,1), infant_data(:,2), infant_data(:,3),'binsx',sx,'binsy',sy);
[xx2, Ni_sm] = smoothheatmap(edgeXi, edgeYi, Ni);

pcolor(xx2{1}, xx2{2}, Ni_sm)
shading('flat')
xlim([0 0.5])
ylim([0 5.5])
caxis([0 10000])
title('Infant F0')
xlabel('VF lengthening')
ylabel('Pressure (kPa)')
hc = colorbar('Ticks',0:5000:18000,'TickLabels',[0,5,10,15]);
hc.Label.String = 'F0 (kHz)';

subplot(1,2,2)
[edgeXa, edgeYa, Na] = nmeanhist_smooth(adult_data(:,1), adult_data(:,2), adult_data(:,3), 'binsx',sx,'binsy',sy);
[xx2, Na_sm] = smoothheatmap(edgeXa, edgeYa, Na);

pcolor(xx2{1}, xx2{2}, Na_sm)
shading('flat')
xlim([0 0.5])
ylim([0 5.5])
caxis([500 10000])
title('Adult F0')
xlabel('VF lengthening')
ylabel('Pressure (kPa)')
hc = colorbar('Ticks',0:5000:18000,'TickLabels',[0,5,10,15]);
hc.Label.String = 'F0 (kHz)';
set(gcf,'position',[-1000,100,1000,400])
colormap(jet)

%%
% plot entropy map
figure(2)
subplot(1,2,1)
sx = 0.75; % moving average factor
sy = 0.5;
[edgeXi, edgeYi, Ni] = nmeanhist_smooth(infant_data(:,1), infant_data(:,2), infant_data(:,4),'binsx',sx,'binsy',sy);
[xx2, Ni_sm] = smoothheatmap(edgeXi, edgeYi, Ni);

pcolor(xx2{1}, xx2{2}, Ni_sm)
shading('flat')
xlim([0 0.5])
ylim([0 5.5])
caxis([-40 -10])
title('Infant WE')
xlabel('VF lengthening')
ylabel('Pressure (kPa)')
hc = colorbar('Ticks',-50:10:0);
hc.Label.String = 'Entropy (dB)';

subplot(1,2,2)
[edgeXa, edgeYa, Na] = nmeanhist_smooth(adult_data(:,1), adult_data(:,2), adult_data(:,4), 'binsx',sx,'binsy',sy);
[xx2, Na_sm] = smoothheatmap(edgeXa, edgeYa, Na);

pcolor(xx2{1}, xx2{2}, Na_sm)
shading('flat')
xlim([0 0.5])
ylim([0 5.5])
caxis([-40 -10])
title('Adult WE')
xlabel('VF lengthening')
ylabel('Pressure (kPa)')
hc = colorbar('Ticks',-50:10:0);
hc.Label.String = 'Entropy (dB)';
set(gcf,'position',[0,100,1000,400])
colormap(jet)

%%
% plot SL map
figure(3)
subplot(1,2,1)
sx = 0.75; % moving average factor
sy = 0.5;
[edgeXi, edgeYi, Ni] = nmeanhist_smooth(infant_data(:,1), infant_data(:,2), infant_data(:,5),'binsx',sx,'binsy',sy);
[xx2, Ni_sm] = smoothheatmap(edgeXi, edgeYi, Ni);

pcolor(xx2{1}, xx2{2}, Ni_sm)
shading('flat')
xlim([0 0.5])
ylim([0 5.5])
caxis([20 70])
title('Infant SL')
xlabel('Vocal fold strain')
ylabel('Pressure (kPa)')
hc = colorbar('Ticks',20:10:70);
hc.Label.String = 'SL (dB)';

subplot(1,2,2)
[edgeXa, edgeYa, Na] = nmeanhist_smooth(adult_data(:,1), adult_data(:,2), adult_data(:,5), 'binsx',sx,'binsy',sy);
[xx2, Na_sm] = smoothheatmap(edgeXa, edgeYa, Na);

pcolor(xx2{1}, xx2{2}, Na_sm)
shading('flat')
xlim([0 0.5])
ylim([0 5.5])
caxis([20 70])
title('Adult SL')
xlabel('Vocal fold strain')
ylabel('Pressure (kPa)')
hc = colorbar('Ticks',20:10:70);
hc.Label.String = 'SL (dB)';
set(gcf,'position',[0,100,1000,400])
colormap(jet)

%%
% plot ME map
figure(4)
subplot(1,2,1)
sx = 0.75; % moving average factor
sy = 0.5;
[edgeXi, edgeYi, Ni] = nmeanhist_smooth(infant_data(:,1), infant_data(:,2), infant_data(:,6),'binsx',sx,'binsy',sy);
[xx2, Ni_sm] = smoothheatmap(edgeXi, edgeYi, Ni);

pcolor(xx2{1}, xx2{2}, Ni_sm)
shading('flat')
xlim([0 0.5])
ylim([0 5.5])
caxis([-70 -50])
title('Infant ME')
xlabel('Vocal fold strain')
ylabel('Pressure (kPa)')
hc = colorbar('Ticks',-70:10:-50);
hc.Label.String = 'ME (dB)';

subplot(1,2,2)
[edgeXa, edgeYa, Na] = nmeanhist_smooth(adult_data(:,1), adult_data(:,2), adult_data(:,6), 'binsx',sx,'binsy',sy);
[xx2, Na_sm] = smoothheatmap(edgeXa, edgeYa, Na);

pcolor(xx2{1}, xx2{2}, Na_sm)
shading('flat')
xlim([0 0.5])
ylim([0 5.5])
caxis([-70 -50])
title('Adult ME')
xlabel('Vocal fold strain')
ylabel('Pressure (kPa)')
hc = colorbar('Ticks',-70:10:-50);
hc.Label.String = 'ME (dB)';
set(gcf,'position',[0,100,1000,400])
colormap(jet)